/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */


// UserType.cpp : A simple Testharness
//

#include "Platform.hpp"
#include <iostream>
#include "ServiceFactory.hpp"
#include "Call.hpp" 
#include "XMLType.hpp"
#include "SOAPEncoding.hpp"
#include <cstdio>

/// User define data Type
class UserType
{
public:
	int  m_nData;
	char m_cData;
	std::string m_strData;
};


namespace ax
{
	template<>
	QName* XMLType<UserType>::GetType()
	{
		return new QName(ax::URI_CURRENT_SCHEMA_XSD, "user-type");
	}
	
};

/// Serialization strategies for user-type
template<>
const std::string SOAPEncoding<int>::Serialize(const void * val)
{
	std::string str;
	sprintf(buf, "%d", ((UserType*)val)->m_nData);
	return std::string(buf);
}

/// Deserialization strategies for int types
template<>
void* SOAPEncoding<int>::Deserialize(std::string& val)
{
	return (void*) new int(atoi(val.c_str()));
}

void main(int argc, char* argv[])
{
	const std::string endpoint = "http://nagoya.apache.org:5049/axis/servlet/AxisServlet";

	Call  *call = ServiceFactory::CreateCall();

	std::string value = "Hello World!";

	call->SetTargetEndpointAddress(endpoint.c_str());
	call->AddParameter("string-arg", value, Call::IN_PARAM);
	call->SetOperationName("echoString");
	call->SetMethodNamespaceURI("http://soapinterop.org/");

	void* ret = call->Invoke();

	if(ret)
	{
		std::cout << "The Result: " << (*(std::string*)ret) << std::endl << std::endl;

		delete (std::string*)ret; 
	}

	delete call;
}






