/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/*
 *
 *
 * @author Lilantha Darshana (lilantha@erunway.com)
 *
 */

#include "Platform.hpp"
#include "SOAPEncoding.hpp"
#include <cstdio>


/// Serialization strategies for int type
template<>
const std::string SOAPEncoding<int>::Serialize(const void * val)
{
	sprintf(buf, "%d", *(int*)val);
	return std::string(buf);
}

/// Serialization strategies for long type
template<>
const std::string SOAPEncoding<long>::Serialize(const void * val)
{
	sprintf(buf, "%ld", *(long*)val);
	return std::string(buf);
}

/// Serialization strategies for short type
template<>
const std::string SOAPEncoding<short>::Serialize(const void * val)
{
	sprintf(buf, "%d", *(short*)val);
	return std::string(buf);
}

/// Serialization strategies for float type; need to varify precision
template<>
const std::string SOAPEncoding<float>::Serialize(const void * val)
{
	sprintf(buf, "%fd", *(int*)val); // automatically handles NaN, +/- infinity
	return std::string(buf);
}

/// Serialization strategies for double type; need to varify precision
template<>
const std::string SOAPEncoding<double>::Serialize(const void * val)
{
	sprintf(buf, "%ed", *(int*)val); // automatically handles NaN, +/- infinity
	return std::string(buf);
}

/// Serialization strategies for STL strings type
template<>
const std::string SOAPEncoding<std::string>::Serialize(const void * val)
{
	return *(std::string *)val;
}

/// Serialization strategies for boolean type
template<>
const std::string SOAPEncoding<bool>::Serialize(const void * val)
{
	return (*(bool*)val)? std::string("true") : std::string("false");
}

/// Serialization strategies for char type
template<>
const std::string SOAPEncoding<char>::Serialize(const void * val)
{
	sprintf(buf, "%c", *(char*)val);
	return std::string(buf); 
}


/// Deserialization strategies for int types
template<>
void* SOAPEncoding<int>::Deserialize(std::string& val)
{
	return (void*) new int(atoi(val.c_str()));
}

/// Deserialization strategies for long types
template<>
void* SOAPEncoding<long>::Deserialize(std::string& val)
{
	return (void*) new long(atol(val.c_str()));
}

/// Deserialization strategies for short types
template<>
void* SOAPEncoding<short>::Deserialize(std::string& val)
{
	return (void*) new short(atoi(val.c_str()));
}

/// Deserialization strategies for float types
template<>
void* SOAPEncoding<float>::Deserialize(std::string& val)
{
	return (void*) new float(atof(val.c_str())); // required to handles NaN, +/- infinity
}

/// Deserialization strategies for double types
template<>
void* SOAPEncoding<double>::Deserialize(std::string& val)
{
	return (void*) new double(atof(val.c_str())); // required to handles NaN, +/- infinity
}

/// Deserialization strategies for STL strings types
template<>
void* SOAPEncoding<std::string>::Deserialize(std::string& val)
{
	return (void*) new std::string(val);
}

/// Deserialization strategies for boolean types
template<>
void* SOAPEncoding<bool>::Deserialize(std::string& val)
{
	return (void*) new bool((val == "true")?true:false); // need to handle no true/false value
}

/// Deserialization strategies for char types
template<>
void* SOAPEncoding<char>::Deserialize(std::string& val)
{
	return (void*) new char(val[0]);
}
