/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/*
 *
 *
 * @author Lilantha Darshana (lilantha@erunway.com)
 *
 */


#if !defined(_AXIS_SOAPENCODING_HPP)
#define _AXIS_SOAPENCODING_HPP


#include "SerializerFactory.hpp"

/**
 * 
 * This is the preimplementation of the SerializerFactory interface for 
 * predefined types. And is used to serialize/deserialize predefined types
 * when encoding and decoding these types from/to SOAP types (xml).
 * If SOAPEncoding<T> is undefined for user defined classes then user have 
 * to implement it. Look in class SOAPEncoding<T> for examples.
 *
 * @brief preimplementation of the SerializerFactory interface for predefined types
 */


template<typename T>
class SOAPEncoding  : public SerializerFactory
{
public:
	SOAPEncoding(){};
	~SOAPEncoding(){};

	/**
	 *	Serialize/converts a C++ object to an XML representation using
	 *	a specific XML processing mechanism and based on the specified type 
	 *	mapping and encoding style.
	 *
	 *	@brief	Serialize/converts a C++ object to an XML representation
	 */
	const std::string Serialize(const void * val);

	/**
	 *	Deserialize/converts an XML representation to a C++ object using 
	 *	a specific XML processing mechanism and based on the specified type 
	 *	mapping and encoding style. 
	 *
	 *	@brief	Deserialize/converts an XML representation to a C++ object
	 */
	void* Deserialize(std::string& val);

private:
	char buf[100]; ///< Temporary buffer to manage encoding/decoding

};

#endif // _AXIS_SOAPENCODING_HPP


