/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/*
 *
 * @author	 Lilantha Darshana	 (lilantha@erunway.com)
 *			
 *
 */


#if !defined(_AXIS_XMLTYPE_HPP)
#define _AXIS_XMLTYPE_HPP

#include "AxisConst.hpp"
#include "QName.hpp"




namespace ax
{

/**
 * 
 * This is the declaration of the XMLTypes which need to be implemented 
 * to manage serialization/deserialization of given types,
 * when encoding and decoding data types from/to SOAP types (xml).
 *
 * If XMLType<T> is undefined for user defined classes then user have 
 * to implement it. Look in definition in XMLType.cpp for examples. 
 * Where QName handle the data type as SOAP schema QName as:
 * xsi:int, xsi:string and so on. The user is responsible for definition of this
 * interface for his/her class (data type) to manage correct schema definition
 * for their type.
 *
 * Eg: For UsrClass, the schema type could be usr_class so the QName is
 *     xsi:usr_class, which should be returned by implementing this interface	
 *
 * Note that this is inlcuded in the namespace ax
 *
 * @brief Declaration of the inteface XMLTypes to obtain xml-schema types as Qualified name
 */

template<typename T>
class XMLType  
{
	public:
		XMLType();
		virtual ~XMLType()=0;

		/// The strategy for returning schema type as Qualified name for the specified type T
		static QName* GetType();
};

};

#endif // _AXIS_XMLTYPE_HPP




