/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/*
 *
 *
 * @author Lilantha Darshana (lilantha@erunway.com)
 *
 */


#if !defined(_AXIS_MESSAGEELEMENT_HPP)
#define _AXIS_MESSAGEELEMENT_HPP

#include "AttList.hpp"
#include <xercesc/dom/DOM_Node.hpp>
#include <xercesc/dom/DOM_Element.hpp>


class XMLSerializer;
class XMLDeSerializer;

/**
 *	Base class of all SOAP message components; components are Envelop, Header, Body
 *	(including RPC element & Fault element), 
 *	There is a one-to-one mapping between MessageElement and DOM-Element for a
 *	given SOAP envelop.
 *	
 * @brief	Base class of all SOAP message components
 *
 */

class MessageElement  
{
	public:
		/// Constructor for the MessageElement
		MessageElement();

		/// Construct from a DOM Element
		MessageElement(const DOM_Element& p_Elm);

		/// Constructor for the MessageElement
		MessageElement(const std::string& p_Ns, const std::string& p_LocalPart);
		virtual ~MessageElement();

		/// Deserialize MessageElement
		virtual void   DeSerialize(XMLDeSerializer& dserilizer){}

		/// Serialize MessageElement
		virtual void   Serialize(XMLSerializer& serilizer){}

		/// Sets MessageElement associated DOM Element.
		virtual void   SetMsgElm(const DOM_Element& p_Elm);

		/// Sets MessageElement Qualified name
		virtual void   SetElmQName(const ax::QName& p_qName){m_qName = p_qName;};

		/// Add any MessageElement as childs by associating corresponded DOM Element
		virtual void   AddMsgElement(const DOM_Element& p_Elm){}

		/// Get the Qualified name of this MessageElement
		virtual ax::QName GetElmQName(){return m_qName;};

	protected:
		std::string    m_Name ;	///< Name or Localpart of this MessageElement
		std::string    prefix ;	///< Namespace prefix of this MessageElement
		std::string    m_NsURI;	///< Namespace URI of this MessageElement

		DOM_Element m_pElmNode;	///< Associated DOM Element.
		AttList		m_AttLst;	///< Associated Arributes.

		ax::QName m_qName;		///< Qualified name of this MessageElement
    
		std::string m_Msg;
};

#endif // _AXIS_MESSAGEELEMENT_HPP


