/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/*
 *
 *
 * @author Lilantha Darshana (lilantha@erunway.com)
 *
 */

#include "Platform.hpp"
#include "RPCElement.hpp"
#include "RPCParam.hpp"
#include "XMLSerializer.hpp"
#include "XMLDeSerializer.hpp"
#include "AxisException.hpp"
#include <xercesc/dom/DOM_Element.hpp>
#include <cstdio>


RPCElement::RPCElement(const std::string& ns, const std::string& methodName,
					   const std::vector<Param*>& params)
{
	m_NsURI = ns;
    m_Name  = methodName;
    
	int nSize = params.size();

    for (int i = 0; i < nSize; i++) 
	{
		std::string name;

		RPCParam *param = new RPCParam(params[i]->m_Name, params[i]->m_pValue, *(params[i]->m_pType));

		m_pParams.push_back(param);
    }
}

RPCElement::RPCElement(const DOM_Element& p_Elm)
{
	SetMsgElm(p_Elm);
	m_Prefix = p_Elm.getPrefix().transcode();
	m_qName = ax::QName(p_Elm.getNamespaceURI().transcode(), 
						p_Elm.getLocalName().transcode());
}


RPCElement::~RPCElement()
{
	int size = m_pParams.size();

	for(int i = 0; i < size;i++)
		delete m_pParams[i];

	m_pParams.clear();
}

void RPCElement::AddMsgElement(const DOM_Element& p_Elm)
{
	SetMsgElm(p_Elm);

	std::string tagName = p_Elm.getTagName().transcode();
	
	if(m_pElmNode.hasChildNodes())
	{
		DOM_Node child=m_pElmNode.getFirstChild();

		if(child.getNodeType() == DOM_Node::TEXT_NODE 
			|| child.getNodeType() == DOM_Node::CDATA_SECTION_NODE)
		{
			// All types are bound to name space URI_CURRENT_SCHEMA_XSI
			std::string schemaType = m_pElmNode.getAttributeNS(ax::URI_CURRENT_SCHEMA_XSI.c_str(), "type").transcode();
			m_Result = child.getNodeValue().transcode();

			RPCParam *result = new RPCParam("result", m_Result, schemaType);
			m_pParams.push_back(result);
		}
	}
	else
		throw AxisException("Return data not found\n\n", 1);
}

void * RPCElement::GetReturnValue()
{
	RPCParam *pRpc = (m_pParams.size()) ? m_pParams[0] : NULL;

	if(pRpc) 
		return pRpc->GetValue();
	
	throw AxisException("Unexpected result from the callee....\n\n", 1);
}

void RPCElement::Serialize(XMLSerializer& serilizer)
{
	ax::QName qName(m_NsURI, m_Name);
	serilizer.StartElem(qName, m_AttLst);

	for (paramItr = m_pParams.begin(); paramItr != m_pParams.end(); ++paramItr) 
	{
		(*paramItr)->Serialize(serilizer);
	}
	serilizer.EndElem();
}

void RPCElement::DeSerialize(XMLDeSerializer& dserilizer)
{
	dserilizer.RegisterPrefixForURI(m_Prefix, m_qName.GetNsURI());

	dserilizer.DeSerialize(*this);
	for (paramItr = m_pParams.begin(); paramItr != m_pParams.end(); ++paramItr) 
	{
		(*paramItr)->DeSerialize(dserilizer);
	}
	
}



