/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/*
 *
 *
 * @author Lilantha Darshana (lilantha@erunway.com)
 *
 */


#include "Platform.hpp"
#include "RPCParam.hpp"
#include "XMLSerializer.hpp"
#include "XMLDeSerializer.hpp"
#include "TypeMappingRegistry.hpp"
#include "SerializerFactory.hpp"
#include <cstdlib>
#include <cstdio>

RPCParam::RPCParam(const std::string& ns, const std::string& name, 
				   void* val, const ax::QName& p_Type)
{
	m_NsURI  = ns;
	m_Name	 = name;
	m_pValue = val;
	m_Type   = p_Type; 
}

RPCParam::RPCParam(const std::string& name, void* val, const ax::QName& p_Type)
{
	m_Name	 = name;
	m_pValue = val;
	m_Type   = p_Type; 
}

RPCParam::RPCParam(const std::string& name, std::string& strValue, std::string& strType)
{
	m_Name	  = name;
	m_Value   = strValue;
	m_strType = strType;
}

RPCParam::~RPCParam()
{

}


void RPCParam::Serialize(XMLSerializer& serilizer)
{
	AttList attribs;

	std::string serVal = serilizer.GetTypeMapper()->GetSerializerFactory(m_Type)->Serialize(m_pValue);

	std::string prefix = serilizer.GetPrefixForURI(m_Type.GetNsURI());
	prefix += ":" + m_Type.GetLocalPart();
	ax::QName qName("", ax::XML_SCHEMA_INSTANCE_TYPE);
	attribs.Add(qName, prefix);
	qName = ax::QName(m_NsURI, m_Name);

	serilizer.Serialize(qName, attribs, serVal.c_str());
}

void RPCParam::DeSerialize(XMLDeSerializer& dserilizer)
{
	m_Type	 = dserilizer.GetQNameFromStr(m_strType);// get as a QName
	m_pValue = dserilizer.GetTypeMapper()->GetSerializerFactory(m_Type)->Deserialize(m_Value);
}
