/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/*
 *
 *
 * @author Lilantha Darshana (lilantha@erunway.com)
 *
 */


#include "Platform.hpp"
#include "SOAPMessage.hpp"
#include "XMLSerializer.hpp"
#include "XMLDeSerializer.hpp"
#include "AxisException.hpp"
#include <iostream>


SOAPMessage::~SOAPMessage()
{

}

const MessageElement * SOAPMessage::GetSOAPBodyElement()
{
	const MessageElement *body = m_envMsg.GetSOAPBodyElement();
	return ((SOAPBodyElement*)body)->GetBody();		
}

const MessageElement* SOAPMessage::GetSOAPHeader()
{
	return NULL;		// TODO: need to return from the envelop
}

const MessageElement* SOAPMessage::GetSOAPEnvelop()
{
	return &m_envMsg;
}


void SOAPMessage::SetSOAPBodyElement(MessageElement *p_pMsgElm)
{
	SOAPBodyElement *body = new SOAPBodyElement(p_pMsgElm);
	m_envMsg.AddSOAPBodyElement(body);
}

void SOAPMessage::SetSOAPHeader(MessageElement *p_pHd)
{

}

void SOAPMessage::SetSOAPEnvelop(MessageElement * p_pEnv)
{

}

void SOAPMessage::Serialize(TypeMappingRegistry* p_TypMapper)
{
	XmlWriter writer;
	XMLSerializer *serilizer = new XMLSerializer(writer);

	serilizer->SetTypeMappingRegistry(p_TypMapper);
	m_envMsg.Serialize(*serilizer);
	m_Msg = writer.str();

	

	try
	{
		delete serilizer;
	}
	catch(...)
	{
		std::cout << "Error on parser termination..." << std::endl;
	}

#ifdef _DEBUG
	std::cout << writer.str();
#endif
}

void  SOAPMessage::SetMsg(const std::string& p_Msg)
{
	// Try to get as SOAP message
	std::string::size_type pos;

	if((pos = p_Msg.find("<")) != std::string::npos)
	{
		m_Msg = p_Msg.substr(pos, p_Msg.rfind(">") - pos + 1);
	}
	else
		throw AxisException("Error in Message ...", 1);
}

void SOAPMessage::DeSerialize(TypeMappingRegistry* p_TypMapper)
{
	//Set the string form of the SOAP envelop for deserialization.
	XMLDeSerializer dserilizer(m_Msg);
	dserilizer.SetTypeMappingRegistry(p_TypMapper);

	try
	{
		//Generate Envelop from a string 
		m_envMsg.RegisterPrefixes(dserilizer);
		dserilizer.DeSerialize(m_envMsg);
		m_envMsg.DeSerialize(dserilizer);
	}
	catch (AxisException& axExp)
	{
		std::cerr << axExp.GetErrorMsg() << std::endl;
		// Clear trace message?? do we want to do this
		axExp.Clear();
		throw;
	}
}

