/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/*
 *
 *
 * @author Lilantha Darshana (lilantha@erunway.com)
 *
 */

#include "Platform.hpp"
#include "SOAPMsgContext.hpp"
#include "Sender.hpp"
#include "Receiver.hpp"
#include "TransportFactory.hpp"
#include "TypeMappingRegistry.hpp"
#include <iostream>



SOAPMsgContext::SOAPMsgContext()
	:m_pTransportChannel(0), m_pReqMsg(0), m_pRepMsg(0)
{
	m_pTypeMpRegistry = new TypeMappingRegistry();
}

SOAPMsgContext::~SOAPMsgContext()
{
	delete m_pTransportChannel;
	delete m_pTypeMpRegistry;
	delete m_pRepMsg;
	delete m_pReqMsg;
}


void SOAPMsgContext::RegisterTypeMapping(const std::string& typname, SerializerFactory * ser)
{
	m_pTypeMpRegistry->AddSerializerFactory(typname, ser);
}

void SOAPMsgContext::Invoke(RPCElement *p_pBody)
{
	try
	{
		m_pReqMsg = new SOAPMessage();

		m_pReqMsg->SetSOAPBodyElement(p_pBody);
		m_pReqMsg->Serialize(m_pTypeMpRegistry);

		m_pTransportChannel = TransportFactory::GetTransport(m_ServiceUrl); 
		m_pTransportChannel->SetProperty(ax::PROP_SOAPACTION, "");

		Sender   request(m_pTransportChannel);
		m_bRecv = request.Send(m_pReqMsg->GetMsg()); 

		Receiver response(m_pTransportChannel);

		if(m_bRecv)
		{
			m_pRepMsg = new SOAPMessage();
			m_pRepMsg->SetMsg(response.Recv());
			m_pRepMsg->DeSerialize(m_pTypeMpRegistry);
		}
	}
	catch(AxisException& ex)
	{
		std::cerr << ex.GetErrorMsg() << std::endl;
		throw;
	}
	catch(...)
	{
		std::cerr << "Unexpected Exception in SOAPMsgContext::Invoke() ....." << std::endl;
		throw;
	}
}

const MessageElement* SOAPMsgContext::GetResults()
{
	return m_pRepMsg->GetSOAPBodyElement();
}

