/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/*
 *
 *
 * @author Lilantha Darshana (lilantha@erunway.com)
 *
 */

#include "Platform.hpp"
#include "NsStack.hpp"


/**
 * Finds namespace URI in the map
 */
const std::string NsMap::findURI(const std::string& uri)
{
	NsMap_t::iterator itr;
	for(itr=begin();itr!=end();++itr)
	{
		if(itr->first == uri) return itr->second;
	}
	return std::string("");
}


/**
 * Find namaspace prefix in the map
 */
const std::string NsMap::findPrefix(const std::string& prefex)
{
	NsMap_t::iterator itr;
	for(itr=begin();itr!=end();++itr)
	{
		if(itr->second == prefex) return itr->first;
	}
	return std::string("");
}

/**
 *	Add namesppace prefix mapping for a URI
 */
void NsStack::Add(const std::string& uri, const std::string& prefix)
{
	if(!Ns.empty())
	{
		NsMap *tmpMap = Peek();
		tmpMap->insert(std::make_pair(uri, prefix));
	}
	else
	{
		NsMap tmpMap;
		tmpMap.insert(std::make_pair(uri, prefix));
		Ns.push_back(tmpMap);
		Indx++;
	}
}

/**
 *	push an empty map on the stack
 */
void NsStack::Push()
{
	NsMap tmpMap;
	tmpMap.insert(std::make_pair(std::string(""), std::string("")));
	Ns.push_back(tmpMap);
	Indx++;
}

/**
 *	Get map at the top of the stack
 */

NsMap* NsStack::Peek() 
{
	if(Ns.empty()) return NULL;
	return &Ns[Indx];
}

/**
 *	Remove a namespace mapping
 */
void NsStack::Pop()
{
	if(Indx>=0)
	{
		Ns.pop_back();
		Indx--;
	}
}

/**
 *	Get a prefix for a given namespace URI
 *
 */

const std::string NsStack::GetPrefix(const std::string& uri)
{
	std::vector<NsMap>::iterator itr;

	for(itr = Ns.begin(); itr != Ns.end(); ++itr)
	{
		std::string prefix = itr->findURI(uri);
		if(!prefix.empty()) return prefix;
	}
	return std::string("");
}

/**
 *	Get a namespace URI for a prefix.
 *
 */
const std::string NsStack::GetNamespaceURI(const std::string& prefix)
{
	std::vector<NsMap>::iterator itr;

	for(itr = Ns.begin(); itr != Ns.end(); ++itr)
	{
		std::string uri = itr->findPrefix(prefix);
		if(!uri.empty()) return uri;
	}
	return std::string("");
}
 
