/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/*
 *
 *
 * @author Lilantha Darshana (lilantha@erunway.com)
 *
 */

#include "Platform.hpp"
#include "QName.hpp"
#include <xercesc/dom/DOM_Element.hpp>
#include "AxisConst.hpp"

namespace ax
{

/**
 *	Constructor for the QName
 */
 
QName::QName(const std::string& p_qName)
{
	if(!p_qName.empty())
	{
		std::string::size_type pos = p_qName.find(':');

		if(pos == std::string::npos)
		{
			m_strLocalPart = p_qName;
		}
		else
		{
			m_strLocalPart = p_qName.substr(pos+1);
			m_strNsURI  = p_qName.substr(0, pos);
		}
	}

}


/**
 *	Constructor for the QName
 */

QName::QName(const std::string& p_strNsPrefix, const std::string& p_strLocalPart)
{
    m_strNsURI  = p_strNsPrefix;
    m_strLocalPart = p_strLocalPart;
}

    
/**
 *	Constructor for the QName
 */

QName::QName(const QName& p_qName)
{
	m_strNsURI  = p_qName.m_strNsURI;
	m_strLocalPart = p_qName.m_strLocalPart;
}


/**
 *	Constructor for the QName
 */
QName::QName(const std::string& p_qName, const DOM_Element& p_Elem) 
{
	if(!p_qName.empty())
	{
		std::string::size_type pos = p_qName.find(':');

		if (pos == std::string::npos) 
		{
            m_strLocalPart = p_qName;
            m_strNsURI	   = "";
        } 
		else 
		{
			m_strLocalPart = p_qName.substr(pos);
			DOMString str  = p_Elem.getAttributeNS(ax::NS_URI_XMLNS.c_str(), 
													p_qName.substr(0, pos).c_str());
            m_strNsURI  = (char *) str.rawBuffer();
        }
	}
}

/**
 *	Sets the Namespace URI for this QName    
 */
void QName::SetNsURI(const std::string& p_strNsURI)
{
	m_strNsURI = p_strNsURI;
}


/**
 *	Sets the Local part for this QName
 */

void QName::SetLocalPart(const std::string& p_strLocalPart)
{
	m_strLocalPart = p_strLocalPart;
}

/**
 *	Returns a string representation of this QName
 *
 */
const std::string QName::toString() 
{
	return (m_strNsURI.empty()) ? m_strLocalPart : (m_strNsURI + ":" + m_strLocalPart);
};
  
/**
 *	Indicates whether some other object is "equal to" this QName object
 */

bool  QName::operator == (const QName& p_qName) 
{
	std::string tmp = ((p_qName.m_strNsURI.empty()) ? p_qName.m_strLocalPart : (p_qName.m_strNsURI + ":" + p_qName.m_strLocalPart));
	return (toString() == tmp);
}

/**
 *	Storage stratergy in a std::map we need a get around for this.
 */
bool operator<(const QName& left, const QName& right)
{
	std::string rstr = ((right.m_strNsURI.empty()) ? right.m_strLocalPart : (right.m_strNsURI + ":" + right.m_strLocalPart));
	std::string lstr = ((left.m_strNsURI.empty()) ? left.m_strLocalPart : (left.m_strNsURI + ":" + left.m_strLocalPart));
	return (lstr < rstr);
}


};
