/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/*
 *
 *
 * @author Lilantha Darshana (lilantha@erunway.com)
 *
 */


#if !defined(_AXIS_QNAME_HPP)
#define _AXIS_QNAME_HPP

#include <string>
#include <list>
#include <vector>



class	DOM_Element;



namespace ax // Put this in the Axis namespace o/w conflicts with other 
{			 // QName definition such as Xerces QName which, is not identical

	/**
	 *	Qualified name based on "Namespaces in XML" specification. defined as follows:
	 *	
	 *	QName	  ::=  (Prefix ':')? LocalPart										<br>
	 *	Prefix	  ::=  NCName														<br>
	 *	LocalPart ::=  NCName														<br>
	 *	NCName	  ::=  (Letter | '_') (NCNameChar)*  ;  An XML Name, minus the ":"	<br>
	 *	NCNameChar ::=  Letter | Digit | '.' | '-' | '_' etc.						<br>
	 *
	 *	The Prefix provides the namespace prefix part of the qualified name, and must 
	 *	be associated with a namespace URI reference in a namespace declaration. 
	 *	The LocalPart provides the local part of the qualified name. 
	 *	Note that the prefix functions only as a placeholder for a namespace name. 
	 *	Applications should use the namespace name, not the prefix, in constructing 
	 *	names whose scope extends beyond the containing document.
	 *
	 *	
	 *	@brief	Qualified name based on "Namespaces in XML" specification
	 */

	 // we need our own Qualified Name class
	class QName
	{
		public:
			/// Constructor for the QName
			QName() : m_strNsURI(""), m_strLocalPart("") {}

			/// Constructor QName from qualified name
			QName(const std::string& p_qName);

			/// Constructor for the QName
			QName(const std::string& p_strNsURI, const std::string& p_strLocalPart);

			/// Constructor for the QName
			QName(const QName& p_qName);

			/// Constructor for the QName
			QName(const std::string& p_qName, const DOM_Element& p_Elem);
			~QName(){};

			/// Sets the Namespace URI for this QName
			void SetNsURI(const std::string& p_strNsURI);

			/// Sets the Local part for this QName
			void SetLocalPart(const std::string& p_strLocalPart);

			/// Gets the Namespace URI for this QName
			const std::string GetNsURI() { return m_strNsURI;}

			/// Gets the Local part for this QName
			const std::string GetLocalPart() {return m_strLocalPart;}
        
			/// Indicates whether some other object is "equal to" this QName object
			bool operator==(const QName& p_qName); 

			/// storage purpose in a std::map we need a get around for this.
			friend bool operator<(const QName& left, const QName& right);

			/// Returns a string representation of this QName
			const std::string QName::toString(); 

		private:
			std::string  m_strNsURI;		///< Namespace URI
			std::string  m_strLocalPart;	///< Loaclpart
	};
};

#endif // _AXIS_QNAME_HPP

