/* -*- C++ -*- */

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "SOAP" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

/*
 *
 *
 * @author Lilantha Darshana (lilantha@erunway.com)
 *
 */


#if !defined(_AXIS_XML_DESERIALIZER_H)
#define _AXIS_XML_DESERIALIZER_H


#include "MessageElement.hpp"
#include <xercesc/dom/DOM.hpp>
#include <xercesc/sax/SAXException.hpp>
#include <xercesc/sax/ErrorHandler.hpp>
#include <xercesc/sax/SAXParseException.hpp>
#include "AxisConst.hpp"
#include "NsStack.hpp"



class  ParseErrorHandler;
class  MessageElement;
class  TypeMappingRegistry;



/**
 *	Delare and define SOAP Envelop Deserializer for deserialization of
 *	XML SOAP envelop; This uses XML DOM parser to parse the SOAP Envelop
 *  and build out a DOM document which contain each components of the 
 *	SOAP Envelop such as SOAP Header, body, RPC components etc.
 * 
 *	@brief  SOAP Envelop deserialiser 
 *
 */

class XMLDeSerializer
{
public:
	/// Create deserializer by parsing the SOAP Envelop passed
	XMLDeSerializer(const std::string& p_Msg);
	~XMLDeSerializer();

	/// Deserialize a given SOAP Message Element
	void DeSerialize(MessageElement& p_Elm);

	/// Sets the current branch that the deserilisation involve.
	void SetContext(DOM_Node&  p_CurNode){m_pCurNode = p_CurNode;}

	/// register prefixes for given URI
	void RegisterPrefixForURI(const std::string& prefix, const std::string& uri);

	/// Convert QName to a string form consisting of prefix:localpart
	std::string QNameToStr(ax::QName& qName);

	/// Convert prefix:localpart to a QName
	ax::QName GetQNameFromStr(std::string strQName);

	/// Sets Type Mapping Registry for type mapping of given encoding
	void SetTypeMappingRegistry(TypeMappingRegistry* p_TypMapper){m_pTypeMapper = p_TypMapper;}

	/// Get a handle to the current type mapping registry
	TypeMappingRegistry * GetTypeMapper(){return m_pTypeMapper;}



private:

	NsStack	m_nsStk;					///< Namespace stack for namespace mapping for the current context

	DOM_Node  m_pCurNode;				///< Current DOM Node of the branch the deserilization apply

	DOMParser *parser;					///< DOM parser to parse the SOAP envelop
	ParseErrorHandler *errReporter;		///< DOM parser error handler
	
	TypeMappingRegistry *m_pTypeMapper;	///< Type mapping registry to obtain deserilized objects.
};




/**
 *	XML parser error handler implementation
 */

class  ParseErrorHandler: public ErrorHandler
{
public:
    void warning(const SAXParseException& e);
    void error(const SAXParseException& e);
    void fatalError(const SAXParseException& e);
    void resetErrors() {};

};


#endif //_AXIS_XML_DESERIALIZER_H

